@@ -1,16 +1,18 @@
-''' Encapsulate a phylogenetic tree space. A phylogenetic landscape or tree space refers to the entire combinatorial space comprising all possible phylogenetic tree topologies for a set of n taxa. The landscape of n taxa can be defined as consisting of a finite set T of tree topologies. Tree topologies can be associated with a fitness function f (t i ) describing their fit. This forms a discrete solution search space and finite graph (T, E) = G. E(G) refers to the neighborhood relation on T(G). Edges in this graph are bidirectional and represent transformation from one tree topology to another by a tree rearrangement operator. An edge between t i and t_j would be notated as e_ij in E(G). '''
+''' Encapsulate a phylogenetic tree space. A phylogenetic landscape or tree space refers to the entire combinatorial space comprising all possible phylogenetic tree topologies for a set of n taxa. The landscape of n taxa can be defined as consisting of a finite set T of tree topologies. Tree topologies can be associated with a fitness function $f(t_i)$ describing their fit. This forms a discrete solution search space and finite graph $(T, E) = G$. $E(G)$ refers to the neighborhood relation on T(G). Edges in this graph are bidirectional and represent transformation from one tree topology to another by a tree rearrangement operator. An edge between $t_i$ and $t_j$ would be notated as $e_{ij}$ in $E(G)$. '''
 
 # Date:   Jan 24 2014
 # Author: Alex Safatli
 # E-mail: safatli@cs.dal.ca
 
-import networkx, rearrangement
+import networkx
 from random import choice
 from scoring import getParsimonyFromProfiles as parsimony, getLogLikelihood as ll
 from parsimony import profile_set as profiles
 from bipartition import bipartition
 from networkx import components as comp, algorithms as alg
-from newick import parser, removeBranchLengths, postOrderTraversal, numberUnrootedTrees
+from tree import tree, treeSet, numberRootedTrees, numberUnrootedTrees
+from newick import parser, removeBranchLengths, postOrderTraversal
+from rearrangement import TYPE_NNI, TYPE_SPR, TYPE_TBR
 
 # Graph Object
 
@@ -23,7 +25,13 @@ class graph(object):
         else: self.graph = gr
         self.defaultWeight = 0.
     
-    def __len__(self):       return len(self.graph.node)
+    def getNetworkXObject(self):
+        
+        ''' Return the internal networkx graph object. '''
+        
+        return self.graph    
+    
+    def __len__(self): return len(self.graph.node)
 
     def getSize(self):
 
@@ -42,6 +50,7 @@ class graph(object):
     def getEdgesFor(self,i): return [self.getEdge(i,j) for j in self.graph.neighbors(i)]
     def getNode(self,i):     return self.graph.node[i] 
     def getEdge(self,i,j):   return self.graph.get_edge_data(i,j)
+    
     def __iter__(self):
         for node in self.graph.node.keys(): yield node
         
@@ -136,9 +145,9 @@ class graph(object):
         
         return alg.shortest_path_length(self.graph,nodA,nodB)
 
-# Landscape: Subclass of Graph Object
+# Landscape: Subclass of Graph, TreeSet Objects
 
-class landscape(graph):
+class landscape(graph,treeSet):
     
     ''' Defines an entire phylogenetic tree space. '''
     
@@ -149,13 +158,13 @@ class landscape(graph):
         super(landscape,self).__init__()
         
         # Fields
-        self.alignment = ali
-        self.locks     = list()
-        self.nextTree  = 0
-        self.leaves    = None
-        self.root      = None
+        self.alignment          = ali
+        self.locks              = list()
+        self.nextTree           = 0
+        self.leaves             = None
+        self.root               = None
+        self.operator           = operator
         self.parsimony_profiles = None
-        self.operator  = operator
         
         # Analyze alignment.
         if ali:
@@ -179,7 +188,11 @@ class landscape(graph):
                 sc = parsimony(new,p)
                 tre.score = (None,sc)            
         
-    def getAlignment(self):           return self.alignment
+    def getAlignment(self): 
+        
+        ''' Acquire the alignment object associated with this space. '''
+        
+        return self.alignment
     
     def getNumberTaxa(self):
         
@@ -204,7 +217,11 @@ class landscape(graph):
         
         return numberUnrootedTrees(self.leaves)
     
-    def getRootTree(self):            return self.root
+    def getRootTree(self):
+        
+        ''' Acquire the first tree that was placed in this space. '''
+        
+        return self.root
     
     def setAlignment(self,ali):
         
@@ -282,14 +299,7 @@ class landscape(graph):
         self.nextTree += 1
         return tree.getName()
 
-    def addTreeByNewick(self,newick):
-        
-        ''' Add a tree to the landscape by Newick string. '''
-        
-        t = tree(newick)
-        self.addTree(t)
-
-    def exploreRandomTree(self,i,type=rearrangement.TYPE_SPR):
+    def exploreRandomTree(self,i,type=TYPE_SPR):
         
         ''' Acquire a single neighbor to a tree in the landscape by performing
         a random rearrangement of type SPR (by default), NNI, or TBR. Rearrangement
@@ -360,7 +370,7 @@ class landscape(graph):
         
         return None        
 
-    def exploreTree(self,i,type=rearrangement.TYPE_SPR):
+    def exploreTree(self,i,type=TYPE_SPR):
         
         ''' Get all neighbors to a tree named i in the landscape using a respective
         rearrangement operator as defined in the rearrangement module. Rearrangement
@@ -526,9 +536,21 @@ class landscape(graph):
 
     # Search Methods
 
+    def __getitem__(self, i): return self.graph.node[i]
+
+    def indexOf(self, tr):
+        
+        ''' Acquire the index/name in this landscape of a tree object. Returns -1
+        if not found. '''
+        
+        for t in self.graph.node:
+            if self.getTree(t) == tr: return t
+        return -1
+
     def findTree(self,newick):
         
-        ''' Find a tree by Newick string, taking into account branch lengths. '''
+        ''' Find a tree by Newick string, taking into account branch lengths. Returns
+        the name of this tree in the landscape. '''
         
         for t in self.graph.node:
             if self.getTree(t).newick == newick: return t
@@ -651,22 +673,25 @@ class landscape(graph):
 
     # Output Methods
     
+    def __str__(self): return self._dump()
+    
     def _dump(self,proper=True):
         
         trees = []
         for t in self.getNodeNames():
             trees.append(self.getVertex(t))
         return '\n'.join([t.getProperNewick() for t in trees])
-    
-    def toTreeFile(self,fout):
+
+    def toTreeSet(self):
         
-        ''' Output this landscape as a series of trees, separated by
-        newlines, as a text file saved at the given path. '''
+        ''' Convert this landscape into an unorganized set of trees 
+        where taxa names are transformed to their original form (
+        i.e. not transformed to a state friendly for the Phylip format). '''
         
-        o = open(fout,'w')
-        o.write(self._dump())
-        o.close()
-        return fout
+        treeset = treeSet()
+        for t in self.getNodeNames():
+            treeset.addTree(tree(self.getVertex(t).getProperNewick()))
+        return treeset
 
 # Comprising Vertices of Landscapes
 
